class EmailManagerError(Exception): ...
class ConfigError(EmailManagerError): ...
class AuthError(EmailManagerError): ...
class SMTPError(EmailManagerError): ...
class IMAPError(EmailManagerError): ...
class ParseError(EmailManagerError): ...
